#include "../xmega/avr_compiler.h"		// Pins und Ports
#include <avr/io.h>
//#include <avr/pgmspace.h>			// strings in flash
#include <string.h>					// strcpy, memcpy
#include "global.h"					// 

#include "clocks.h"					// F_TICK, F_SEC


// Kommunikation //////////////////////////////////////////////////////////

/*	Interfaces:

	UART 
		rsx		RS485-dux	USARTC0			-> ETH, USB
		sci		ABIC-SCI	USARTE0			-> Anybus: CAN, PB ...

	SPI 
		pot		Potis		Master 	SPIC 	-> Potentiometer
		ram		RAM			Master 	SPID 	-> MRAM 4 Mb
		ssc		ABIC-SSC	Slave 	SPIE 	-> Anybus
*/


// Transmit universal ////////////////////////////////////////////////
/*
// Transmit universal SPI und UART
void 	putch(uint8_t byte);	// noch nicht fr SPI
void 	putx(uint8_t *array, uint16_t anz);
void 	putstr(char *string);
*/

void putch(uint8_t byte) {			// fr UART verwendbar, 
	// nutzt globale Variablen fr USART Status- und Datenregister
	// wait until transmit buffer is empty (data register empty flag - DREIF)
	while (!(*REG.statusreg & REG.txflag)) { }	// _bm data register empty
	*REG.datareg = byte;		// write byte to uart
}


inline void putx(uint8_t *array, uint16_t anz) {	// little endian, X86
	while (anz > 0)	{ 		// send hex byte array, lsb first
		putch(*array++); 	// least significant byte first
		anz--; 
	}
	bufsig = ready;
}


void putstr(char *string)	{	// sende char null-terminierten string
	while (*string) {				// while string is true
		putch(*string++);
	}
	bufsig = ready;
}



// Receive universal ////////////////////////////////////
/*
// Receive universal SPI und UART
uint8_t getch(void); 
void 	getx(uint8_t *rxbuffer, uint8_t anz); 
void 	getstr(uint8_t *rxbuffer, uint8_t buflen);
*/


uint8_t getch(void) {	// RXCIF wird gelscht, wenn RX-Buffer leer ist p.249
	while (!(*REG.statusreg & REG.rxflag)) {;} 	// warte bis Bedingung wahr ist ###
	return *REG.datareg;						// Byte holen
}


inline void getx(uint8_t *rxbuffer, uint8_t anz) {	// hex-Block, interruptfhig 
	// gets braucht ca. 66 Zyklen. Bei fCPU = 32 MHz und 
	// 4 Mb/s = 400 kHz pro Byte stehen nur 80 Takte zur Verfgung: Vorsicht!
	uint8_t nextchar;
	nextchar = getch();
	static uint8_t bufnum;

	while(	bufnum < (anz-1) ) 
	{
		*rxbuffer++ = nextchar;
		bufnum++;
		nextchar = getch();
	}
	bufnum = 0;		// fertig
	bufsig = string_complete;
}


void getstr(uint8_t *rxbuffer, uint8_t buflen) {	// ASCII-Block, interruptfhig 
	// gets braucht ca. 66 Zyklen. Bei fCPU = 40 MHz und 
	// 5 Mb/s = 500 kHz pro Byte stehen 80 Takte zur Verfgung: Vorsicht!
	uint8_t nextchar;					
	nextchar = getch();
	static uint8_t bufnum;


	while( (nextchar != '\r') && (bufnum < buflen-1) ) 
	{
		*rxbuffer++ = nextchar;
		bufnum++;
		nextchar = getch();
	}
	*rxbuffer = '\r';	// Terminierung abweichend von C und Unix
	bufnum = 0;			// fertig
	bufsig = string_complete;
}



// DEBUG u.a. /////////////////////////////////////////////////////////////

void debugOut(uint16_t value) {	// sendet einen Wert als ASCII
	// first task: Vorwort ausgeben
	putstr((char *) "\nDebug-Wert = ");
	// second task: Wert ausgeben
	while(value > 0) {
		char temp = (value % 10) + '0';
		value = (value / 10);
		putch(temp);
	}
}


void i2asc(uint16_t value) {	// integer to ascii
	while(value > 0) {
		char temp = (value % 10) + '0';
		value = (value / 10);
		putch(temp);
	}
}




//////////////////////////////////////////////////////////////
// 		STRUKTUREN etc.
//////////////////////////////////////////////////////////////


// REC ///////////////////////////////////////////////////////

// REC_struct init	(6 Byte + CH*2 byte hex)	 CH = 13: frame = 32 Byte)
void RECinit(void) {		// Init-Werte fr ADC 12 bit

	REC.FS = 0x47;			// Framesync ('G'), Identifier + Framesynchronisation
	REC.ID = PRL.ID;		// Identifier vom Parametersatz
	REC.TS = 0x0000;		// time stamp
	REC.SEC = 0x00;			// current second
	REC.CH = 13;			// Kanalzahl
}


// Test REC_struct (16 Worte a 2 Byte = 32 Byte)
void genTestRec(void) {		// Init-Werte fr ADC 12 bit

	REC.FS = 0x47;			// Framesync, Identifier und Framesynchronisation
	REC.ID = 0x00;			// Identifier
	REC.TS = 0x0000;			// Time stamp, sample (wird mit systick incrementiert)
	REC.SEC = 0x00;			// Second
	REC.CH = 13;				// Kanalzahl

	REC.K1 = 0x4b31;			// 1 	int16, 12 bit right aligned, positiv
	REC.K2 = 0x4b32;			// 2	dito
	REC.K3 = 0x4b33;			// 3
	REC.K4 = 0x4b34;			// 4
	REC.K5 = 0x4b35;			// 5
	REC.K6 = 0x4b36;			// 6
	REC.KU = 0x4b55;			// U	int16, 12 bit right aligned, negativ
	REC.KI = 0x4b49;			// I	dito

	REC.RA = 0x5241;			// A
	REC.RB = 0x5242;			// B
	REC.RC = 0x5243;			// C
	REC.RD = 0x5244;			// D
	REC.RE = 0x5245;			// E
}


// Parameter fr Factory Reset (par_t struct) ////////////////////////////////////

void genPar(void) {	// Zugriff z.B. PT.P1 = 0x00e3, PT.ID = 0x2639
						// hex-Parameter im EEPROM 32 x 16 bit = 512 Bit = 64 Byte
		// diese wird nach Reset ins RAM geladen und bei Burn ins EEPROM gebrannt 

	PRL.ID = 0x4e;			// Ident.-Nr. Parametersatz frei whlbar
	PRL.SEC= 0x00;			// zu sendende Sekunden: speziell: PRL.SEC = 0 --> Preview mit PRL.FT
	PRL.AO = 0xff; 			// Autooffset on/off [7...0]; channels [8...1]
	PRL.NE = 0x00; 			// Negation bits NEG[7...0]; channels [8...1]

	PRL.FT = 0x0258; 		// 600 Systicks pro Preview ~ 75ms
	PRL.FH = F_SEC; 		// Anzahl von Systicks pro Sekunde aus clocks.h (read only)
	PRL.NA = 0x0001; 		// Nummer des Weld-Algorithmus
	PRL.NP = 0x0000; 		// frei

	PRL.P1 = 0x0033;		// gain for potentiometer channel 1...n, right adjust --> '0' 'f'
	PRL.P2 = 0x0034;		// 0x66 means from DAC 2x4bit to ADC-input 1,6V in theory
	PRL.P3 = 0x0035;
	PRL.P4 = 0x0036;
	PRL.P5 = 0x0037;
	PRL.P6 = 0x0038;
	PRL.PU = 0x0039; 		// Gain U
	PRL.PI = 0x003a; 		// Gain I
	
	PRL.O1 = 0x0000;		// offset '0' signed = 0x0000, left adjust
	PRL.O2 = 0x0000;		// 			
	PRL.O3 = 0x0000;
	PRL.O4 = 0x0000;
	PRL.O5 = 0x0000;
	PRL.O6 = 0x0000;
	PRL.OU = 0x0000;		// offset channel U
	PRL.OI = 0x0000;		// offset channel I
							// DAC-Hub: 12 Bit ==> 0...4096 = 0...3,3V
	PRL.VA = 0x076c; 		// DAC-output VREF A -> ARA (right adj)  1,65 Volt ~ 0x800 ~ dez 2048
	PRL.VB = 0x076c; 		// DAC-output VREF B -> ARB (right adj)	offset shift ("131 8")

	PRL.G1 = 0x0000;		// 
	PRL.T1 = 0x0064; 		// 100  STOP-Dauer, z.B. 100/8 ~ 12,5 ms
	PRL.G2 = 0x0000;		// 
	PRL.T2 = 0x0000; 		// 1   Pulsverlngerung T2
	PRL.G3 = 0x0000;		// 
	PRL.T3 = 0x0000; 		// 
	PRL.TA = 0x0000; 		// 0	Schwellwert fr START welding
	PRL.TE = 0xff38; 		// -200	Schwellwert fr STOP welding
};


void genFrame(void) {	// Frame mit Parameterliste fllen

	FR.PRL = PRL;		// Parameterliste bernehmen
	
	FR.SRC[0] = 'S';
	FR.SRC[1] = 'R';
	FR.SRC[2] = 'C';
	FR.SRC[3] = ':';
	
	FR.DST[0] = 'D';
	FR.DST[1] = 'S';
	FR.DST[2] = 'T';
	FR.DST[3] = ':'; 
	
	memcpy(&FR.MAC, "ionMAC", 6);	
	memcpy(&FR.DAT, version, strlen(version));
}


void genError(void) {	// Fehlermeldung ausgeben
	memcpy(&FR.DAT, error, strlen(error));
}

/////////////////////////////////////////////////////////////////
// DATEN IM FLASH ///////////////////////////////////////////////
/////////////////////////////////////////////////////////////////


// Strings ////////////////////////////////////////////////////////





