#include "../xmega/avr_compiler.h"		// Pins und Ports
#include "spi.h"
#include "global.h"
#include "leds.h"
#include "clocks.h"					// toggleTickpin

/********************************************************************************/
// General SPI ******************************************************************
/********************************************************************************/


/*  SPI_t ist in iox192a3.h vereinbart ber #include <avr/iox192a3.h>
// oder automatisch ber:
#define mcu atxmega192a3			// ldt ber avr_compiler und iom die Registernamen
#define F_CPU 32000000UL			// mu vor avr_compiler.h stehen
#include <xmega/avr_compiler.h>		// ldt wichtige Namen
*/


/* // gilt fr alle I/O-Ports PINnCTRL:
#define TOTEMPOLE 		0b00000000	// OPC = 000, A-manual p.142 (default)
#define BUSKEEPER		0b00001000	// OPC = 001
#define PULLDOWN		0b00010000	// OPC = 010, input with pull down
#define PULLUP 			0b00011000	// OPC = 011, input with pull up
#define WIRANDPULL 		0b00111000	// OPC = 111, output wired AND with pull up
#define WIRAND	 		0b00101000	// OPC = 101, output wired AND

// gilt fr alle SPI-Interfaces im IO-Space: 
#define nCS				(1<<PIN4)	// SPI /SS
#define MOSI			(1<<PIN5)	// SPI  SI
#define MISO			(1<<PIN6)	// SPI  SO
#define SCK				(1<<PIN7)	// SPI  CLK

// gilt fr alle SPI-Interfaces CTRL:
#define SPI_CLKPER2		0b10000000	// CLKPER/2, A-manual p.233; 
#define SPI_CLKPER4		0b00000000	// CLKPER/4, A-manual p.233; 
#define SPI_CLKPER8		0b10000001	// CLKPER/8, A-manual p.233; 
#define SPI_CLKPER16	0b00000001	// CLKPER/16, A-manual p.233; 

// #define SPI_ENA		0b01000000	// enable SPI; bit 6 = 1 -> SPI_ENABLE_bm
*/


/********************************************************************************/
// SPI Tx ///////////////////////////////////////////////////////////////////
// with global defines for datareg and statusreg


void putc_SPI(uint8_t sbyte) { 		// bitte SPI vorher initialisieren: statusreg, datareg
	*(REG.datareg) = sbyte;			// send data byte to SPI-PORT
	// Wait for transmission complete:
	loop_until_bit_is_set (*(REG.statusreg), SPI_IF_bp);	// warte bis fertig ###
	// SPI-Interrupt-Flag SPIF: A-manual p.230/233}
	// loop_until_bit_is_set siehe <avr/sfr_defs.h>: Special function registers
}


void putw_SPI(uint16_t word) {					// byte order: high, low
	uint8_t lowbyte = (uint8_t) word;			// cast von 16 auf 8 low
	uint8_t highbyte = (uint8_t) (word>>8);		// cast and shift to low
	putc_SPI(highbyte);
	putc_SPI(lowbyte);
}


void putx_SPI(uint8_t *array, uint8_t numof) {	// put a number of hex bytes
	// numof 	number of bytes to be transmitted
	// *array	pointer to hex array (not terminated)
	while (numof > 0) {
		numof--;
		putc_SPI(*array++);
	}
	bufsig = ready;
}


/********************************************************************************/
// SPI Rx ///////////////////////////////////////////////////////////////////
// with global defines for datareg and statusreg

uint8_t getc_SPI(void) {	// bitte SPI vorher initialisieren: statusreg, datareg
	while (!(*(REG.statusreg) & SPI_IF_bm)) {;}	// warte bis nchstes Zeichen fertig ###
	return *(REG.datareg);						// send data byte to SPI-PORT
}


uint16_t getw_SPI(void) {					// getword byteorder: high, low
	uint16_t word = (uint16_t) getc_SPI();	// erstes Byte in word schreiben
	word = word<<8;							// um acht Stellen nach links schieben
	word |= (uint16_t) getc_SPI();			// zweites Byte einfgen
	return word;
}


void getx_SPI(uint8_t *array, uint8_t numof) {		// get a number of hex bytes
	// numof 	number of bytes to be transmitted
	// return	hex byte output ( not terminated)
	while (numof > 0) { 
		numof--;
		*array++ = getc_SPI();
	}
	bufsig = string_complete;
}



/********************************************************************************/
// SPI Potentiometer ************************************************************
/********************************************************************************/


	// ATxmega is SPI-master; 
	// MCP-potis are SPI-slaves in daisy-chain config.

	// Signal	Pin		Function
	// nSS		PC4		/CS chip select output to MCP42xxx - chain
	// MOSI		PC5		Master output to MCP42xxx
	// MISO		PC6		Master input from MCP42xxx
	// SCK		PC7		SCK clock to MCP42xxx - chain

	// Poti Microchip MCP42xxx: IC hat zwei Potis
	// die richtige Reihenfolge bestimmt die Ankunft:
	// letztes Poti (K5 und K6) zuerst senden
	// erst command byte, dann data byte, msb first
	// Daisy-Chain Configuration,
	// Timing siehe MCP42xxx Datenblatt, p.19, Fig.5-3

	// MCP42 command byte (register cmd):
	//	0b_xxxx_xx10	fr Poti mit Pins 5,6,7	-> K1,K3,K5
	//	0b_xxxx_xx01 	fr Poti mit Pins 8,9,10 ->	K2,K4,K6
	//	0b_xxxx_xx11	both potis (K12, K34, K56) - not in use.
	//
	//	0b_xx01_xxxx	write data
	//	0b_xx10_xxxx	shutdown

	// Wert 0x00 bedeutet 0 Ohm zwischen Pins 5-6 bzw.9-10

/*
#define POTPORT PORTC				// I/Os port C							###
#define POTSPI  SPIC				// SPI port C							###

#define POTG	0x11				// Command fr MCP42 "Poti gerade"		###
#define POTU	0x12				// Command fr MCP42 "Poti ungerade"	###

#define POTRATE SPI_CLKPER64		// A-manual p.232; Poti max. freq. 5 MHz

*/
void SPI_POTIconfig() {		// SPI fr Potis einstellen, PORT C, master
	// SPI_t see iox192a3.h
	POTPORT.OUTSET |= nCS;	// Chip Select deaktivieren /CS = high
	// SPI-PORT/PIN-Config siehe A-manual, p.230, table 20-1 !!!
	// SPI-MASTER: only MISO is defined as input, other pins "user defined" !
	// Set input pin direction (low)			unntig
	// POTPORT.DIR &= ~MISO;		// unntig, siehe #defines
	// Set outputs pin direction (high)
	POTPORT.DIR |= (nCS |  MOSI | SCK);	// siehe #defines
	// set pull-up resistor for master input
	POTPORT.PIN6CTRL |= PULLUP;				// MISO PIN position ###
	// set clock rate, set order, set edge dir:
	// p5 = 0 data order: msb first; p4 = 1 master mode, 
	// p32 = 00 rising edge; p10 = 01 prescaler clkper/16 (2 MHz)
	// A-manual p.232; Poti max. freq. 5 MHz (MCP042-manual, p.5)
	POTSPI.CTRL |= POTRATE;				// Datenrate (we are busmaster)
	POTSPI.CTRL |= SPI_MASTER_bm;		// enable SPI as busmaster
}

void sendPotiGains() {	// input: struct PRL.xx (global)
	display(red);

	// stellt drei kettengeschaltete Potentiometer vom
	// Typ: Microchip MCP42050
	// richtige Reihenfolge bestimmt die Ankunft:
	// letztes Poti (K5, K6) zuerst senden
	//
	// MCP42 command byte (register cmd):
	//	0b_xxx1_xx10	0x12	fr Poti mit Pins 5,6,7		K1,3,5	POTU ungerade
	//	0b_xxx1_xx01 	0x11	fr Poti mit Pins 8,9,10	K2,4,6	POTG gerade
	//	0b_xxx1_xx11	0x13	both potis (for chopper)
	//	0b_xx01_xxxx	write data
	//	0b_xx10_xxxx	shutdown
	// Wert 0x00 bedeutet 0 Ohm zwischen Pins 5-6 bzw.9-10

	// enable SPI, b6 = 1
	POTSPI.CTRL |= SPI_ENABLE_bm;

	// drei Potis liegen jeweils hintereinander, siehe Schaltplan
	// zuerst das letzte Poti einstellen, zum Schlu das Erste

	// erster Schub (gerade Potis)
		POTPORT.OUTCLR |= nCS;			// /CS = low, enable
		putc_SPI( POTG);				// gerades Poti stellen (0x11)
		putc_SPI((uint8_t)  PRL.P6);	// MCP42 command byte Poti_0, pins 8-9-10
		putc_SPI( POTG);
		putc_SPI((uint8_t)  PRL.P4);	// cast von 16 auf 8 bit
		putc_SPI( POTG);				// ungerades Poti stellen (0x12)
		putc_SPI((uint8_t)  PRL.P2);
		POTPORT.OUTSET |= nCS;			// /CS = high, disable -> bernahme

	// gengend lange warten:
	display(toggle);
	toggleTickpin();
	display(toggle);

	//zweiter Schub (ungerade Potis)
		POTPORT.OUTCLR |= nCS;			// /CS = low, enable
		putc_SPI( POTU);
		putc_SPI((uint8_t)  PRL.P5);	// MCP42 command byte Poti_1, pins 5-6-7
		putc_SPI( POTU);
		putc_SPI((uint8_t)  PRL.P3);
		putc_SPI( POTU);
		putc_SPI((uint8_t)  PRL.P1);	
		POTPORT.OUTSET |= nCS;			// /CS = high, disable -> bernahme


	// disable SPI
	POTSPI.CTRL &=~ SPI_ENABLE_bm;

	display(green);	// wait with /CS=1 for 1sec (see spec potis, dt > 40ns)
} // putch doesnt work for poti-spi



/********************************************************************************/
// AnyBus-IC ABIC-SSC, SPI als SLAVE ********************************************
/********************************************************************************/


// SSC-ABIC-Interface fr CAN, Profibus, Ethernet, USB 

	/* ABIC-SSC: 
	The ABIC expects that any circuitry that is wired to the SSC is 100% sequence- 
	and timing-compatible to a shift register chain built with 165/594 discrete shift registers.

	Outputs of ABIC: /RESET, DO, CLK, nLD (!) -> ABIC-SSC works as BUSMASTER! ATXmega is SLAVE.
	Inputs: DI1 and DI2

	- SSC_DO (ABIC-Modul, Pin 4)
	Serial data output. Valid on the falling edge of SSC_CLK.
	(It not used, leave this signal unconnected).

	- SSC_DI2 (ABIC-Modul, Pin 5)
	Serial data input 1 from shift registers. Sampled on the rising edge of SSC_CLK.
	(It not used, leave this signal unconnected).

	- SSC_DI1 (ABIC-Modul, Pin 6)
	Serial data input 2 from shift registers. Sampled on the rising edge of SSC_CLK.
	- If only using Input Registers, connect this signal to SCC_DO.
	- If only using Output Registers, connect this signal to SCC_DI2.
	- If using both Input- and Output Registers, connect this signal to the serial data line between
	the Input- and Output Registers.
	(It not used, leave this signal unconnected).

	- SSC_CLK (ABIC-Modul, Pin 7)
	Clock output.
	(It not used, leave this signal unconnected).

	- /SSC_Reset_Out (ABIC-Modul, Pin 2)
	Active low signal used to reset the shift registers. The use of this signal is optional.
	That this signal must not be confused with /RESET (ABIC-Modul, Pin 8).
	(It not used, leave this signal unconnected).

	- /SSC_LD (ABIC-Modul, Pin 3)
	Shift register load. Loads the value of the shift register inputs on transition to low.
	(If not used, leave this signal unconnected). 
	
	*/

void SSC_SPI_config() {			// ABIC SPI-Interface SSC --- SLAVE
	// PORT
	SSCPORT.OUTSET 	 |= nCS;		// /CS = high, input /SSC_LD off (Xmega: nSS):
	SSCPORT.DIR 	 |= MISO;	// Set data direction to output
	SSCPORT.DIR 	 &= ~(nCS | MOSI | SCK);	// Set slave inputs
	SSCPORT.PIN3CTRL |= PULLUP;				// set pull-up resistor at input	###
	SSCPORT.PIN5CTRL |= PULLUP;				// set pull-up resistor at input	###
	SSCPORT.PIN7CTRL |= PULLUP;				// set pull-up resistor at input	###
	// SPI
	SSCSPI.CTRL 	 |= SPI_CLKPER16;		// set clockrate, see A-manual p.232
	SSCSPI.CTRL 	 &= ~SPI_MASTER_bm;		// set SPI as SLAVE (!!!)
	SSCSPI.INTCTRL 	 |= SPI_INTLVL_LO_gc;	// enable low level interrupt, cleared with access to data register
	SSCSPI.CTRL 	 |= SPI_ENABLE_bm;		// enable SPI
}	


// ABIC-SSC FIFO buffer **********************************************
/*
volatile uint8_t fifoa[FIFOleng];	// Array fr kleines FIFO, length fifolen
volatile uint8_t adrn = 0;			// temporrer Zhler fr FIFO


// ABIC-SSC-Schieberegister, pro Zugriff ein Byte rein, eines raus
uint8_t fifo(uint8_t inbyte) {		// first in, first out
									// input wie output je ein Byte
									// Achtung: Zellenindex 0...(n-1) bei n Werten
	uint8_t outbyte = fifoa[adrn];	// erst Speicherzelle lesen, 
	fifoa[adrn] = inbyte;			// dann dieselbe Zelle neu beschreiben
	if (++adrn >= FIFOleng)			// Adresse erhhen bei jedem Aufruf
		adrn = 0;					// von vorn
	return outbyte;					// gibt outbyte zurck
};
	// testweise Daten spiegeln		### 
	//SPIE.DATA = fifo(SPIE.DATA);	// FIFO dazwischenschalten, Parameter: fifolen ###
*/





